/// <reference types="Cypress" />
/* globals cy, expect */

context("Solid-event automatic tests", () => {
  before(() => {
    cy.clearLocalStorageSnapshot();
  });
  beforeEach(() => cy.restoreLocalStorage());
  afterEach(() => cy.saveLocalStorage());
  it("should login", () => cy.userLogin());

  const eventsList = "section#default-listevents";
  const eventDetail = "section#default-eventdetail";

  // EVENTS LIST DISPLAYED
  it("Access / Events list", () => {
    cy.get('solid-route[name="default-listevents"]').should(
      "have.attr",
      "active"
    );
    cy.get(eventsList).should("not.have.attr", "hidden");
    cy.get("solid-link").should("have.attr", "next", "default-newevent");
    cy.get("solid-display").should("have.attr", "next", "default-eventdetail");
  });

  // CONTENT DISPLAYED OF AN EVENT IN THE LIST EVENT
  it("List elements content", () => {
    cy.get('solid-display[next="default-eventdetail"]>div>solid-display')
      .children()
      .children()
      .should("have.attr", "name", "segment1");
    cy.get('solid-display[next="default-eventdetail"] > div')
      .children()
      .should("have.length", 3);
    cy.get(
      'solid-display[filtered-by="default-solid-event-type-search-future"] > div'
    )
      .children()
      .should("have.length", 2);
    cy.get('solid-display[next="default-eventdetail"] > div > solid-display')
      .eq(1)
      .children()
      .find("solid-set-default")
      .within(() => {
        cy.get("solid-display-img")
          .get('solid-display-value[name="name"]')
          .contains("Futur Event")
          .get('solid-set-default[name="startSet"]')
          .find("span")
          .should("contain", "01.10.2026")
          .get('solid-set-default[name="startSet"]')
          .find("div")
          .should("contain", "12:00")
          .get('solid-set-default[name="endSet"]')
          .find("span")
          .should("contain", "01.10.2026")
          .get('solid-set-default[name="endSet"]')
          .find("div")
          .should("contain", "17:00")
          .get('event-display-type[name="type"]')
          .contains("coaching Session 26 November 2020")
          .get('solid-display-value[name="shortDescription"]')
          .contains("Post event test")
          .get('event-display-location-events[name="location"]')
          .find("a")
          .contains("Palais des congrès")
          .get('event-display-location-events[name="location"]')
          .find("a")
          .should(
            "have.attr",
            "href",
            "https://www.openstreetmap.org/search?query=12, rue du Bac, 75007 Paris"
          );
      });
  });

  //CHECK RESEARCH OPTION
  it("Research with type event", () => {
    cy.get(
      "solid-form-multipleselect-autocompletion-label > solid-form-dropdown > select"
    ).select("Conference", { force: true });
    cy.get('solid-display[next="default-eventdetail"] > div')
      .children()
      .should("have.length", 1);
    cy.get('solid-display[next="default-eventdetail"] > div > solid-display')
      .children()
      .find("solid-set-default")
      .within(() => {
        cy.get('event-display-type[name="type"]').contains("Conference");
      });
    cy.get(
      "solid-form-multipleselect-autocompletion-label > solid-form-dropdown"
    )
      .find(".ss-value-delete")
      .click();

    cy.get(
      "solid-form-multipleselect-autocompletion-label > solid-form-dropdown > select"
    ).select("coaching Session 26 November 2020", { force: true });
    cy.get('solid-display[next="default-eventdetail"] > div')
      .children()
      .should("have.length", 1);
    cy.get('solid-display[next="default-eventdetail"] > div > solid-display')
      .children()
      .find("solid-set-default")
      .within(() => {
        cy.get('event-display-type[name="type"]').contains(
          "coaching Session 26 November 2020"
        );
      });
    cy.get(
      "solid-form-multipleselect-autocompletion-label > solid-form-dropdown"
    )
      .find(".ss-value-delete")
      .click();

    cy.get(
      "solid-form-multipleselect-autocompletion-label > solid-form-dropdown > select"
    ).select("Coaching session 12 January 2021", { force: true });
    cy.get('solid-display[next="default-eventdetail"] > div')
      .children()
      .should("have.length", 1);
    cy.get(
      "solid-form-multipleselect-autocompletion-label > solid-form-dropdown"
    )
      .find(".ss-value-delete")
      .click();
  });

  // DETAILS OF AN EVENT
  it("Details of an event", () => {
    cy.get('solid-display[next="default-eventdetail"] > div > solid-display')
      .eq(0)
      .click();
    cy.get("solid-router")
      .find('solid-route[name="default-eventdetail"]')
      .should("have.attr", "active");
    cy.get(eventsList).should("have.attr", "hidden");
    cy.get(eventDetail).should("not.have.attr", "hidden");
    cy.get('section[data-view="default-eventdetail"]')
      .children()
      .filter("solid-display")
      .should("have.length", 1);
    cy.get('section[data-view="default-eventdetail"] > solid-display > div')
      .find('solid-display-div-multiline[name="longDescription"]')
      .contains("A new event long description");
    // GO BACK TO LIST EVENTS
    cy.get(
      eventDetail + ' > div > solid-link[next="default-listevents"]'
    ).click();
    cy.get(eventDetail).should("have.attr", "hidden");
    cy.get(eventsList).should("not.have.attr", "hidden");
  });

  // PAST EVENTS
  it("Past events list", () => {
    cy.get(
      'solid-display[filtered-by="default-solid-event-type-search-past"] > div'
    )
      .children()
      .should("have.length", 1);
    //DISPLAYED INFO
    cy.get(
      'solid-display[filtered-by="default-solid-event-type-search-past"] > div > solid-display'
    )
      .eq(0)
      .children()
      .find("solid-set-default")
      .within(() => {
        cy.get("solid-display-img")
          .get('solid-display-value[name="name"]')
          .contains("Evenement test")
          .get('solid-set-default[name="startSet"]')
          .find("span")
          .should("contain", "26.11.2020")
          .get('solid-set-default[name="startSet"]')
          .find("div")
          .should("contain", "09:00")
          .get('solid-set-default[name="endSet"]')
          .find("span")
          .should("contain", "26.11.2020")
          .get('solid-set-default[name="endSet"]')
          .find("div")
          .should("contain", "18:00")
          .get('event-display-type[name="type"]')
          .contains("Conference")
          .get('solid-display-value[name="shortDescription"]')
          .contains("test d'évènement")
          .get('event-display-location-events[name="location"]')
          .find("a")
          .contains("Bataclan")
          .get('event-display-location-events[name="location"]')
          .find("a");
      });

    //DETAILS PAST EVENT
    cy.get(
      'solid-display[filtered-by="default-solid-event-type-search-past"] > div > solid-display'
    )
      .eq(0)
      .click();
    cy.get(
      'section[data-view="default-eventdetail"] > solid-display > div'
    ).find('solid-display-div-multiline[name="longDescription"]');
    cy.get(
      eventDetail + ' > div > solid-link[next="default-listevents"]'
    ).click();
  });
});
