import style from "@styles/index.scss?inline";
import * as utils from "@helpers/utils";

import(/* @vite-ignore */ utils.coreVersion()).then((core) => {
  customElements.define(
    "solid-" + utils.componentName,
    class extends core.SolidTemplateElement {
      constructor() {
        super();
        this.setTranslationsPath(`${utils.path()}/locales`);
        utils.setupOrbit(this);
      }

      static get propsDefinition() {
        return {
          dataSrc: "data-src",
          pastEvents: "past-events",
          visibility: "visiblecheckbox",
          visibilityregions: "visibilityregions",
          regionRange: "range-event-region",
          typeRange: "range-event-type",
          circles: "range-event-circle",
          uploadDir: "upload-dir",
          nestedField: "nested-field",
          extraContext: "extra-context",
          postUri: "post-uri",
          postTypesUri: "post-types-uri",
          postRegionUri: "post-region-uri",
          uniq: "uniq",
          displayMode: "displayMode",
          routeEvent: "route-event",
        };
      }

      get extraContext() {
        return JSON.stringify({
          account: "hd:account",
          avatar: "foaf:depiction",
          messageSet: "https://cdn.startinblox.com/owl#message_set",
          author: "https://cdn.startinblox.com/owl#author_user",
          title: "https://cdn.startinblox.com/owl#title",
          ...JSON.parse(this.props.extraContext || "{}"),
        });
      }

      context() {
        return `
            <script id="${this.route}-event-context" type="application/ld+json">
              ${this.extraContext}
            </script>
          `;
      }

      connectedCallback() {
        this.addEventListener("widgetRendered", async (e) => {
          if (!e.target.classList.contains("type-form-search")) return;
          const selector = this.querySelector(
            "solid-form-multipleselect-autocompletion-label"
          );
          const hiddenSelector = Array.from(
            this.querySelectorAll(".hidden-solid-form-search")
          );
          hiddenSelector.forEach((sel) =>
            sel.dispatchEvent(new Event("formChange"))
          );
          selector.addEventListener("input", (event) => {
            const hiddenOptions = Array.from(
              hiddenSelector.map((sel) =>
                Array.from(sel.querySelectorAll("option"))
              )
            ).flat();
            const values = event.target.component.getValue();
            hiddenOptions.forEach((opt) => (opt.selected = false));
            values.forEach((value) =>
              hiddenOptions
                .filter((o) => JSON.parse(o.value)["@id"] === value["@id"])
                .forEach((o) => (o.selected = true))
            );
            hiddenSelector.forEach((sel) =>
              sel.dispatchEvent(new Event("formChange"))
            );
          });

          //FIXME: This is a workaround to force triggering the date filters on component populate - Please rewrite
          setTimeout(async () => {
            const filters = core.Helpers.asyncQuerySelectorAll(
              "solid-form-search",
              this
            );
            for await (const filter of filters) {
              filter.dispatchEvent(new Event("input"));
            }
            this.populate();
          }, 1000);
        });
        this.addEventListener("click", (e) => {
          if (e.target && e.target.closest(".pagination nav")) {
            let display = e.target.closest("solid-display");
            display.scrollIntoView(true);
          }
        });
      }

      routeurs() {
        let render = `
            <solid-router default-route="${this.route}-list">
              <solid-route class="d-none" name="${this.route}-list"></solid-route>
              <solid-route use-id name="${this.route}-detail"></solid-route>
              <solid-route name="${this.route}-create"></solid-route>
              <solid-route use-id name="${this.route}-edit"></solid-route>
            </solid-router>
          `;
        return render;
      }

      loader(name, html = true) {
        let render = `loader-${name}-${this.uniq}`;
        if (html) {
          render = `
            <div id="loader-${name}-${this.uniq}" class="loader loader-top">
              <div></div>
              <div></div>
              <div></div>
              <div></div>
            </div>
            `;
        }
        return render;
      }

      preflightCheck(requirements) {
        // Improve me: global instance and generate only one detailled error
        for (let requirement of requirements) {
          if (requirement === undefined) {
            console.error(
              "Can't initialize Event Component. Missing attributes. Please check documentation."
            );
            return true;
          }
        }
        return false;
      }

      viewWidgets() {
        let render = `
            <solid-widget name="simplelist-event-display-startdate-events-${
              this.uniq
            }">
              <template>
                <span>\${new Date(await value.toString()).toLocaleDateString('fr-FR')}</span>
              </template>
            </solid-widget>
            <solid-widget name="simplelist-event-display-starttime-events-${
              this.uniq
            }">
              <template>
                <span>\${value.split(':').splice(0,2).join(':')}</span>
              </template>
            </solid-widget>
            <solid-widget name="event-display-location-events-${this.uniq}">
              <template>
                <div class="event-place-name">
                  <a
                    href="\${await (value.address ? value.address.then(a => {
                    if(!a) return '#'
                    else if((new RegExp('^https?://')).test(a)) return a
                    else return 'https://www.openstreetmap.org/search?query='+a
                  }) : '#')}"
                    class="margin-bottom-xsmall text-color-secondary icon icon-location-pin icon-margin-right-xxsmall$\{value ? '' : 'hidden'}"
                    target="\${await (value.address ? value.address.then(a => a ? '_blank' : '') : '')}">
                    \${await value.name}
                  </a>
                </div>
              </template>
            </solid-widget>
            <solid-widget name="event-display-startdate-events-${this.uniq}">
              <template>
                <div class="margin-bottom-xxsmall">
                  ${this.localize("label.startdate")}
                  <span
                    class="margin-left-xsmall icon icon-calendar icon-margin-right-xxsmall"
                    >\${new Date(await
                    value.toString()).toLocaleDateString('fr-FR')}</span>
                </div>
              </template>
            </solid-widget>
            <solid-widget name="event-display-enddate-events-${this.uniq}">
              <template>
                <div>
                  ${this.localize("label.enddate")}
                  <span
                    class="margin-left-xsmall icon icon-calendar icon-margin-right-xxsmall"
                    >\${new Date(await
                    value.toString()).toLocaleDateString('fr-FR')}</span>
                </div>
              </template>
            </solid-widget>
            <solid-widget name="event-display-starttime-events-${this.uniq}">
              <template>
                <div class="icon icon-clock icon-margin-right-xxsmall">
                  \${value.split(':').splice(0,2).join(':')}
                </div>
              </template>
            </solid-widget>
            <solid-widget name="event-display-type-${this.uniq}">
              <template>
                <div class="event-type">\${await value.name}</div>
              </template>
            </solid-widget>
            <solid-widget name="event-display-link-event-${this.uniq}">
              <template>
                <a
                  href="\${value}"
                  target="_blank"
                  class="text-color-secondary event-display-link-event icon icon-link icon-margin-right-xxsmall \${value ? '' : 'hidden'}"
                  >\${value.replace(/(https?:\\/\\/[\\w.-]+)\\/.*/, '$1')}</a>
              </template>
            </solid-widget>
            <solid-widget name="event-multiple-label-ifnotvoid-${this.uniq}">
              <template>
                <label class="\${value ? '' : 'hidden'}">\${label}</label>
                <solid-display data-src="\${value}" fields="name"></solid-display>
              </template>
            </solid-widget>
            <solid-widget name="event-edit-${this.uniq}">
              <template>
                <solid-ac-checker permission="acl:Write" data-src="\${value}">
                  <div class="segment full margin-bottom-small margin-top-medium">
                    <solid-link
                      data-src="\${value}"
                      next="${this.route}-edit"
                      class="segment sm-full button text-xsmall text-bold text-uppercase text-center reversed color-secondary bordered icon icon-pencil icon-margin-right-xxsmall">
                      ${this.localize("label.edit_event")}
                    </solid-link>
                  </div>
                  <div class="segment full margin-bottom-small">
                    <solid-delete
                      class="segment sm-full button text-xsmall text-bold text-uppercase text-center color-secondary bordered"
                      data-src="\${value}"
                      data-label="${this.localize("label.delete_event")}"
                      next="${this.route}-list"
                      confirmation-message="${this.localize(
                        "confirm.delete_event"
                      )}">
                    </solid-delete>
                  </div>
                </solid-ac-checker>
              </template>
            </solid-widget>
            <solid-widget name="event-form-starttime-label-${this.uniq}">
              <template>
                <label> ${this.localize("label.start_time")}* </label>
                <input
                  data-holder=""
                  type="time"
                  value="\${value.split(':').splice(0,2).join(':')}"
                  name="startTime"
                />
              </template>
            </solid-widget>
            <solid-widget name="event-form-endtime-label-${this.uniq}">
              <template>
                <label> ${this.localize("label.end_time")}* </label>
                <input
                  data-holder=""
                  type="time"
                  value="\${value.split(':').splice(0,2).join(':')}"
                  name="endTime"
                />
              </template>
            </solid-widget>
          `;
        return render;
      }

      viewSimpleList() {
        return `
            <div class="solid-event">
              <h2 class="text-xxlarge">${this.localize("simplelist.title")}</h2>
              ${this.loader("events-dm-simpleList")}
              <solid-display
                class=""
                data-src="${this.sourceUrl}"
                extra-context="${this.route}-event-context"
                paginate-by="5"
                fields="segment(segment1(startSet(startDate)), segment2(name))"
                class-segment="segment full flex-row"
                class-segment1="segment quarter sm-full text-center whitespace-normal padding-left-xsmall padding-right-xxsmall"
                class-segment2="segment auto sm-full text-top whitespace-normal padding-right-xxsmall"
                widget-name="a"
                widget-startDate="simplelist-event-display-startdate-events-${
                  this.uniq
                }"
                widget-startTime="simplelist-event-display-starttime-events-${
                  this.uniq
                }"
                loader-id="${this.loader("events-dm-simpleList", false)}"
                next="${this.routeEvent}-detail"
              ></solid-display>
              <div class="text-right">
                <solid-link
                  next="${this.routeEvent}"
                  class="segment text-xsmall text-bold text-right reversed color-secondary bordered margin-left-xxlarge margin-top-small text-uppercase">
                  <span class="icon-in-link icon icon-xsmall icon-arrow-right-circle"></span>&nbsp;<span class="link">${this.localize(
                    "simplelist.linktoall"
                  )}</span>
                </solid-link>
              </div>
            </div>
          `;
      }

      viewListEvents() {
        if (this.preflightCheck([this.props.typeRange])) {
          return `Error while loading Event Details. Missing range-event-type attribute.`;
        }
        let render = `
            <section class="listevents bg-color-grey padding-bottom-xlarge" data-view="${
              this.route
            }-list" id="${this.route}-list" no-render="">
              <div class="segment full padding-large sm-padding-top-small sm-padding-right-xsmall sm-padding-bottom-small sm-padding-left-xsmall border-bottom border-color-grey sm-whitespace-normal">
                <div class="segment half sm-full">
                  <h2 class="text-color-heading text-uppercase text-xlarge text-letter-spacing-large">
                    ${this.localize("title.events")}
                  </h2>
                </div>
                <div class="segment half sm-full text-right">
                  <solid-link
                    class="segment button text-xsmall text-bold text-uppercase text-center reversed color-secondary bordered icon icon-plus icon-margin-right-xxsmall"
                    next="${this.route}-create">
                    ${this.localize("label.post_event")}
                  </solid-link>
                </div>
              </div>
              <div class="segment full padding-large sm-padding-xsmall sm-padding-top-normal whitespace-normal">
                <div class="segment half sm-full">
                  <h3 class="text-color-heading text-semibold text-letter-spacing-large">
                    ${this.localize("title.search_event")}
                  </h3>
                  <solid-form-search
                    id="${this.route}-solid-event-type-search-future"
                    class="hidden-solid-form-search"
                    fields="type, endDate"
                    widget-endDate="solid-form-rangedate"
                    start-value-endDate="today"
                    widget-type="solid-form-multipleselect"
                    range-type="${this.props.typeRange}"
                  ></solid-form-search>
                  <solid-form-search
                    id="${this.route}-solid-event-type-search"
                    class="segment block whitespace-normal form"
                    range-type="${this.props.typeRange}"
                    order-asc-type="name"
                    fields="type"
                    class-type="segment margin-bottom-small full sm-full sm-padding-none text-small select-shadow select-bg-white type-form-search"
                    label-type="${this.localize("search.by_type")}"
                    widget-type="solid-form-multipleselect-autocompletion-label"
                  ></solid-form-search>
                </div>
              </div>
              ${this.loader("events")}
              <solid-display
                class="segment full children children-third sm-children-full children-margin-bottom-medium sm-children-margin-bottom-xsmall children-padding-right-large children-padding-left-large sm-children-padding-none sm-whitespace-normal masonry counter pagination"
                data-src="${this.sourceUrl}"
                extra-context="event-context"
                fields="segment1(img, name, startSet(startDate,startTime), endSet(endDate,endTime), type, shortDescription, location, link)"
                filtered-by="${this.route}-solid-event-type-search-future"
                class-segment1="segment hover bg-color-white shadow full text-top text-center whitespace-normal padding-bottom-medium"
                class-img="event-children-bg-color-primary"
                class-name="segment block whitespace-normal text-uppercase text-semibold text-color-heading text-xxlarge text-letter-spacing-larger margin-medium"
                class-startSet="segment block text-bold"
                class-startDate="segment margin-right-xsmall"
                class-startTime="segment margin-left-xsmall"
                class-endSet="segment block text-bold margin-bottom-medium"
                class-endDate="segment margin-right-xsmall"
                class-endTime="segment margin-left-xsmall"
                class-type="text-uppercase tag color-primary bordered"
                class-shortDescription="segment block whitespace-normal margin-small"
                widget-img="solid-display-img"
                widget-startDate="event-display-startdate-events-${this.uniq}"
                widget-startTime="event-display-starttime-events-${this.uniq}"
                widget-endDate="event-display-enddate-events-${this.uniq}"
                widget-endTime="event-display-starttime-events-${this.uniq}"
                widget-type="event-display-type-${this.uniq}"
                widget-shortDescription="solid-display-value-markdown"
                widget-link="event-display-link-event-${this.uniq}"
                widget-location="event-display-location-events-${this.uniq}"
                paginate-by="9"
                next="${this.route}-detail"
                loader-id="${this.loader("events", false)}"
              ></solid-display>
          `;
        if (this.props.pastEvents != "hidden") {
          let yesterday = new Date(
            new Date().setDate(new Date().getDate() - 1)
          );
          let yesterdayFormat =
            yesterday.getFullYear() +
            "-" +
            ("0" + (yesterday.getMonth() + 1)).slice(-2) +
            "-" +
            ("0" + yesterday.getDate()).slice(-2);
          render += `
              <div class="${
                this.props.pastEvents || ""
              } border-top border-color-grey margin-top-medium bg-color-white">
                <solid-form-search
                  id="${this.route}-solid-event-type-search-past"
                  class="hidden-solid-form-search"
                  fields="type, endDate"
                  widget-endDate="solid-form-rangedate"
                  end-value-endDate="${yesterdayFormat}"
                  widget-type="solid-form-multipleselect"
                  range-type="${this.props.typeRange}"
                ></solid-form-search>
                <h3 class="text-color-heading text-semibold text-letter-spacing-large margin-top-xxlarge margin-bottom-medium padding-left-xlarge sm-padding-left-xsmall">
                  ${this.localize("title.past-events")}
                </h3>
                ${this.loader("events-past")}
                <solid-display
                  class="segment full children children-third sm-children-full children-margin-bottom-medium sm-children-margin-bottom-xsmall children-padding-right-large children-padding-left-large sm-children-padding-none sm-whitespace-normal masonry counter pagination"
                  data-src="${this.sourceUrl}"
                  extra-context="event-context"
                  fields="segment1(img, name, startSet(startDate,startTime), endSet(endDate,endTime), type, shortDescription, location, link)"
                  filtered-by="${this.route}-solid-event-type-search-past"
                  class-segment1="segment hover bg-color-grey  border-all-sides border-color-grey shadow full text-top text-center whitespace-normal padding-bottom-medium"
                  class-img="event-children-bg-color-primary"
                  class-name="segment block whitespace-normal text-uppercase text-semibold text-color-heading text-xxlarge text-letter-spacing-larger margin-medium margin-bottom-xsmall"
                  class-startSet="segment block text-bold"
                  class-startDate="segment margin-right-xsmall"
                  class-startTime="segment margin-left-xsmall"
                  class-endSet="segment block text-bold margin-bottom-medium"
                  class-endDate="segment margin-right-xsmall"
                  class-endTime="segment margin-left-xsmall"
                  class-type="text-uppercase tag color-primary bordered"
                  class-shortDescription="segment block whitespace-normal margin-small"
                  widget-img="solid-display-img"
                  widget-startDate="event-display-startdate-events-${this.uniq}"
                  widget-startTime="event-display-starttime-events-${this.uniq}"
                  widget-endDate="event-display-enddate-events-${this.uniq}"
                  widget-endTime="event-display-starttime-events-${this.uniq}"
                  widget-type="event-display-type-${this.uniq}"
                  widget-link="event-display-link-event-${this.uniq}"
                  widget-location="event-display-location-events-${this.uniq}"
                  order-desc="endDate"
                  paginate-by="9"
                  next="${this.route}-detail"
                  loader-id="${this.loader("events-past", false)}"
                ></solid-display>
              </div>
            `;
        }
        render += `
            </section>
          `;
        return render;
      }

      viewEventDetails() {
        return `
            <section data-view="${this.route}-detail" id="${
          this.route
        }-detail" class="eventdetail margin-bottom-xxlarge" no-render="">
            ${this.loader("details-event")}
              <solid-display
                fields="name, info(img, infotext(location, startSet(startDate,startTime), endSet(endDate,endTime), type, circle, shortDescription, link, @id)), longDescription"
                bind-resources
                class="container-1200"
                class-name="event-title segment block text-center text-xxlarge text-uppercase text-bold margin-xxlarge text-color-heading whitespace-normal"
                class-info="segment full margin-bottom-xxsmall sm-whitespace-normal sm-margin-none"
                class-img="segment half sm-full whitespace-normal event-children-bg-color-primary text-center padding-small"
                class-infotext="segment half sm-full whitespace-normal text-top margin-none margin-top-medium margin-left-medium margin-bottom-medium sm-margin-none sm-margin-top-medium sm-margin-bottom-medium sm-padding-small"
                class-location="segment block margin-bottom-small"
                class-startSet="segment block margin-bottom-small"
                class-startDate="segment margin-right-small"
                class-startTime="segment"
                class-endSet="segment block margin-bottom-small"
                class-endDate="segment margin-right-small"
                class-endTime="segment"
                class-shortDescription="segment block whitespace-normal margin-bottom-small"
                class-type="segment margin-bottom-xsmall margin-top-xsmall text-uppercase tag color-primary bordered"
                class-circle="segment block "
                class-longDescription="segment block whitespace-normal margin-top-medium margin-bottom-medium padding-small"
                widget-img="solid-display-img"
                widget-startDate="event-display-startdate-events-${this.uniq}"
                widget-startTime="event-display-starttime-events-${this.uniq}"
                widget-endDate="event-display-enddate-events-${this.uniq}"
                widget-endTime="event-display-starttime-events-${this.uniq}"
                widget-type="event-display-type-${this.uniq}"
                widget-id="event-edit-${this.uniq}"
                widget-circle="event-multiple-label-ifnotvoid-${this.uniq}"
                label-circle="${this.localize("label.linked_circle")}"
                widget-link="event-display-link-event-${this.uniq}"
                widget-location="event-display-location-events-${this.uniq}"
                widget-shortDescription="solid-display-value-markdown"
                widget-longDescription="solid-display-value-markdown"
                loader-id="${this.loader("details-event", false)}"
              ></solid-display>
              <div class="container-1200">
                <solid-link
                  next="${this.route}-list"
                  class="segment button text-xsmall text-bold text-uppercase text-center reversed color-secondary bordered margin-left-small margin-top-medium icon icon-arrow-left icon-margin-right-xxsmall">
                  ${this.localize("button.back_to_list")}
                </solid-link>
              </div>
            </section>
          `;
      }

      viewEventCreate() {
        if (
          this.preflightCheck([
            this.postUri,
            this.props.typeRange,
            this.props.postTypesUri,
            this.props.uploadDir,
          ])
        ) {
          return `Error while loading Event Create.`;
        }
        let render = `
            <section data-view="${this.route}-create" id="${
          this.route
        }-create" class="newevent" no-render="">
              <div class="segment full padding-large sm-padding-top-small sm-padding-right-xsmall sm-padding-bottom-small sm-padding-left-xsmall border-bottom border-color-grey bg-color-white">
                <div class="segment half sm-full">
                  <h2 class="margin-none text-color-heading text-uppercase text-xlarge text-letter-spacing-large">${this.localize(
                    "title.create_new_event"
                  )}</h2>
                </div>
                <div class="segment half sm-hidden text-right">
                  <solid-link
                    class="backlink"
                    bind-resources=""
                    next="${this.route}-list"
                    >${this.localize("title.create_backlink")}</solid-link>
                </div>
              </div>
              <div class="segment full padding-large sm-padding-xsmall sm-padding-top-medium whitespace-normal bg-color-white">
                ${this.loader("new-event")}
                <solid-form
                  class="form"
                  data-src="${this.postUri}"
                  fields="name, startDate, startTime, type, endDate, endTime,${
                    this.props.visibilityregions != "hidden" ? " region," : ""
                  } ${
          this.props.visibility != "hidden" ? " visible," : ""
        } img, shortDescription, longDescription, location.name, location.address, link${
          this.props.circles ? ", circle" : ""
        }"
                  class-name="segment margin-bottom-medium full sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-type="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-startDate="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-startTime="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-endDate="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-endTime="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-img="segment block center margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-shortDescription="segment margin-bottom-medium full sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-longDescription="segment margin-bottom-medium full sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-location.name="segment margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-location.address="segment margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-link="segment margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
          `;
        if (this.props.circles) {
          render += `
                  class-circle="segment margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  range-circle="${this.props.circles}"
                  widget-circle="solid-form-placeholder-dropdown-label"
                  label-circle="${this.localize("label.circle")}"
                  placeholder-circle="${this.localize("label.select_circle")}"
            `;
        }
        if (this.props.visibility != "hidden") {
          render += `
                  class-visible="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal ${
                    this.props.visibility || ""
                  }"
                  widget-visible="solid-form-checkbox-label"
                  label-visible="${this.localize("label.visible")}"
            `;
        }
        if (this.props.visibilityregions != "hidden") {
          render += `
                  class-region="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal ${
                    this.props.visibilityregions || ""
                  }"
                  range-region="${this.props.regionRange}"
                  addable-region-data-src="${this.props.postRegionUri}"
                  addable-region-fields="name"
                  addable-region-placeholder-name="${this.localize(
                    "label.add_event_region"
                  )}"
                  addable-region-widget-name="solid-form-text-placeholder"
                  addable-region-submit-button="${this.localize(
                    "button.save_event_region"
                  )}"
                  addable-region-no-result="${this.localize(
                    "label.event_region_no_result"
                  )}"
                  placeholder-region="${this.localize(
                    "label.select_region_of_event"
                  )}"
                  widget-region="event-solid-form-dropdown-label-addable"
                  label-region="${this.localize("label.region_of_event")}*"
            `;
        }
        render += `
                  widget-name="solid-form-label-text"
                  widget-startDate="solid-form-date-label"
                  widget-startTime="event-form-starttime-label-${this.uniq}"
                  value-startTime="12:00"
                  widget-endDate="solid-form-date-label"
                  widget-endTime="event-form-endtime-label-${this.uniq}"
                  value-endTime="12:00"
                  range-type="${this.props.typeRange}"
                  widget-type="event-solid-form-dropdown-label-addable"
                  addable-type-data-src="${this.props.postTypesUri}"
                  addable-type-fields="name"
                  addable-type-placeholder-name="${this.localize(
                    "label.add_event_type"
                  )}"
                  addable-type-widget-name="solid-form-text-placeholder"
                  addable-type-submit-button="${this.localize(
                    "button.save_event_type"
                  )}"
                  addable-type-no-result="${this.localize(
                    "label.event_type_no_result"
                  )}"
                  placeholder-type="${this.localize(
                    "label.select_type_of_event"
                  )}"
                  widget-img="solid-form-image-label"
                  widget-shortDescription="solid-form-richtext-label"
                  maxlength-shortDescription="150"
                  widget-longDescription="solid-form-richtext-label"
                  label-img="${this.localize("label.image")}"
                  label-startDate="${this.localize("label.start_date")}*"
                  label-startTime="${this.localize("label.start_time")}*"
                  label-endDate="${this.localize("label.end_date")}*"
                  label-endTime="${this.localize("label.end_time")}*"
                  label-type="${this.localize("label.type_of_event")}*"
                  label-name="${this.localize("label.event_name")}*"
                  label-shortDescription="${this.localize(
                    "label.short_description"
                  )}*"
                  label-longDescription="${this.localize(
                    "label.long_description"
                  )}*"
                  label-location.name="${this.localize("label.location")}*"
                  label-location.address="${this.localize("label.address")}"
                  label-link="${this.localize("label.link")}"
                  required-startDate
                  required-startTime
                  required-endDate
                  required-endTime
                  required-name
                  required-shortDescription
                  required-longDescription
                  required-location.name
                  required-type
                  pattern-name=".{1,50}"
                  title-name="${this.localize("validation.name_no_more")}"
                  pattern-shortDescription=".{1,250}"
                  title-shortDescription="${this.localize(
                    "validation.shortDescription_no_more"
                  )}"
                  pattern-link=".{0,2048}"
                  title-link="${this.localize("validation.link_no_more")}"
                  upload-url-img="${this.props.uploadDir}"
                  value-img="${utils.path()}/img/defaulteventtrans.svg"
                  data-js-type="event"
                  loader-id="${this.loader("new-event", false)}"
                  next="${this.route}-list"
                  submit-button="${this.localize("button.publish_event")}"
                  class-submit-button="segment float-right button text-xsmall text-bold text-uppercase text-center reversed color-secondary bordered"
                ></solid-form>
              </div>
            </section>
          `;
        return render;
      }

      viewEventEdit() {
        if (this.preflightCheck([this.props.typeRange, this.props.uploadDir])) {
          return `Error while loading Event Edit.`;
        }
        let render = `
            <section data-view="${this.route}-edit" id="${
          this.route
        }-edit" class="editevent" no-render="">
              <div class="segment full padding-large sm-padding-top-small sm-padding-right-xsmall sm-padding-bottom-small sm-padding-left-xsmall border-bottom border-color-grey bg-color-white">
                <div class="segment half sm-full">
                  <h2 class="margin-none text-color-heading text-uppercase text-xlarge text-letter-spacing-large">
                    <solid-display bind-resources fields="name"></solid-display>
                  </h2>
                </div>
                <div class="segment half sm-hidden text-right">
                  <solid-link
                    class="backlink"
                    bind-resources=""
                    next="${this.route}-detail"
                    >${this.localize("title.edit_backlink")}</solid-link>
                </div>
              </div>
              <div class="segment full padding-large sm-padding-xsmall sm-padding-top-medium whitespace-normal bg-color-white">
                ${this.loader("edit-event")}
                <solid-form
                  class="form"
                  bind-resources
                  fields="name, startDate, startTime, type, endDate, endTime,${
                    this.props.visibilityregions != "hidden" ? " region," : ""
                  } ${
          this.props.visibility != "hidden" ? " visible," : ""
        } img, shortDescription, longDescription, location.name, location.address, link${
          this.props.circles ? ", circle" : ""
        }"
                  class-name="segment margin-bottom-medium full sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-type="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-startDate="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-startTime="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-endDate="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-endTime="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-img="segment block center margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-shortDescription="segment margin-bottom-medium full sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-longDescription="segment margin-bottom-medium full sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-location.name="segment margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-location.address="segment margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-link="segment margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  class-submit-button="segment float-right button text-xsmall text-bold text-uppercase text-center reversed color-secondary bordered"
          `;
        if (this.props.circles) {
          render += `
                  class-circle="segment margin-bottom-medium half sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal"
                  range-circle="${this.props.circles}"
                  widget-circle="solid-form-placeholder-dropdown-label"
                  label-circle="${this.localize("label.circle")}"
                  placeholder-circle="${this.localize("label.select_circle")}"
            `;
        }
        if (this.props.visibility != "hidden") {
          render += `
                  class-visible="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal ${
                    this.props.visibility
                  }"
                  widget-visible="solid-form-checkbox-label"
                  label-visible="${this.localize("label.visible")}"
            `;
        }
        if (this.props.visibilityregions != "hidden") {
          render += `
                  class-region="segment margin-bottom-medium third sm-full padding-right-small sm-padding-none text-small text-semibold text-uppercase text-color-heading whitespace-normal ${
                    this.props.visibilityregions
                  }"
                  range-region="${this.props.regionRange}"
                  widget-region="solid-form-placeholder-dropdown-label"
                  placeholder-region="${this.localize(
                    "label.select_region_of_event"
                  )}"
                  label-region="${this.localize("label.region_of_event")}"
            `;
        }
        render += `
                  widget-name="solid-form-label-text"
                  widget-startDate="solid-form-date-label"
                  widget-startTime="event-form-starttime-label-${this.uniq}"
                  widget-endDate="solid-form-date-label"
                  widget-endTime="event-form-endtime-label-${this.uniq}"
                  range-type="${this.props.typeRange}"
                  widget-type="solid-form-placeholder-dropdown-label"
                  placeholder-type="${this.localize(
                    "label.select_type_of_event"
                  )}"
                  widget-img="solid-form-image-label"
                  widget-shortDescription="solid-form-richtext-label"
                  maxlength-shortDescription="150"
                  widget-longDescription="solid-form-richtext-label"
                  label-img="${this.localize("label.image")}"
                  label-startDate="${this.localize("label.start_date")}*"
                  label-endDate="${this.localize("label.end_date")}*"
                  label-type="${this.localize("label.type_of_event")}*"
                  label-name="${this.localize("label.event_name")}*"
                  label-shortDescription="${this.localize(
                    "label.short_description"
                  )}*"
                  label-longDescription="${this.localize(
                    "label.long_description"
                  )}*"
                  label-location.name="${this.localize("label.location")}*"
                  label-location.address="${this.localize("label.address")}"
                  label-link="${this.localize("label.link")}"
                  required-startDate
                  required-startTime
                  required-endDate
                  required-endTime
                  required-name
                  required-shortDescription
                  required-longDescription
                  required-location.name
                  required-type
                  pattern-name=".{1,50}"
                  title-name="${this.localize("validation.name_no_more")}"
                  pattern-shortDescription=".{1,250}"
                  title-shortDescription="${this.localize(
                    "validation.shortDescription_no_more"
                  )}"
                  pattern-link=".{0,2048}"
                  title-link="${this.localize("validation.link_no_more")}"
                  upload-url-img="${this.props.uploadDir}"
                  data-js-type="event"
                  loader-id="${this.loader("edit-event", false)}"
                  next="${this.route}-detail"
                  submit-button="${this.localize("button.save_event")}"
                ></solid-form>
              </div>
            </section>
          `;
        return render;
      }

      template(props) {
        // Both modes require a data-src
        if (
          this.preflightCheck([
            props.dataSrc || props.hasOwnProperty("bindResources"),
          ])
        ) {
          return `Error while loading Event Component. Missing data-src or bind-resources attribute.`;
        }
        // When visibility regions is not hidden and not in special display mode, regionRange is required
        if (props.visibilityregions != "hidden" && !props.displayMode) {
          if (this.preflightCheck([props.regionRange])) {
            return `Error while loading Event. Missing range-event-region attribute. Either add visibilityregions="hidden" or a proper source.`;
          }
        }

        // FIXME: Too much dependancy on uri format
        this.sourceUrl =
          props.dataSrc + (props.nestedField ? props.nestedField + "/" : "");

        this.postUri =
          (props.postUri || props.dataSrc) +
          (props.nestedField ? props.nestedField + "/" : "");

        this.uniq = props.uniq || utils.uniq();
        if (window.orbit && !props.displayMode) {
          this.route = this.getRoute(this.uniq);
        } else {
          this.route = this.uniq;
        }
        if (window.orbit) {
          this.routeEvent = props.routeEvent || this.getRoute("events", true);
        } else {
          this.routeEvent = props.routeEvent || "events";
        }

        // Both modes will have their own context and widgets
        let render = `
            <style>${style}</style>
            ${this.context()}
            ${this.viewWidgets()}
          `;

        switch (props.displayMode) {
          case "simpleList":
            render += this.viewSimpleList();
            break;
          default:
            if (!props.postTypesUri) {
              this.props.postTypesUri = props.typeRange;
            }
            if (!props.postRegionUri) {
              this.props.postRegionUri = props.regionRange;
            }
            render += `
                ${this.routeurs()}
                <div class="solid-event">
                  ${this.viewListEvents()}
                  ${this.viewEventDetails()}
                  ${this.viewEventEdit()}
                  ${this.viewEventCreate()}
                </div>
              `;
        }

        return render;
      }

      _camelize(str) {
        return str.replace(/[-]+(.)/g, (match, chr) => chr.toUpperCase());
      }

      _uncamelize(str) {
        return str.replace(
          /[A-Z]+/g,
          (match, chr) => "-" + match.toLowerCase()
        );
      }
    }
  );
});
