import SlimSelect from "slim-select";
import * as utils from "@helpers/utils";

import(/* @vite-ignore */ utils.coreVersion()).then((core) => {
  const EventAddableMixin = {
    name: "event-addable-mixin",
    initialState: {
      slimSelect: null,
      mutationObserver: null,
    },
    created() {
      this.slimSelect = null;
      this.addToAttributes(true, "autocomplete");
      this.listCallbacks.push(this.addCallback.bind(this));
    },
    detached() {
      if (this.slimSelect) this.slimSelect.destroy();
      if (this.mutationObserver) this.mutationObserver.disconnect();
    },
    addCallback(value, listCallbacks) {
      if (this.slimSelect) return;
      let select = this.element.querySelector("select");
      if (select) {
        this.initSlimSelect(select);
      } else {
        // if no select available, wait for one and init slimSelect
        new MutationObserver((_m, observer) => {
          const select = this.element.querySelector("select");
          if (select) {
            this.initSlimSelect(select);
            observer.disconnect(); // then disconnect mutationObserver
          }
        }).observe(this.element, {
          childList: true,
          characterData: true,
          subtree: true,
        });
      }
      const nextProcessor = listCallbacks.shift();
      if (nextProcessor) nextProcessor(value, listCallbacks);
    },
    initSlimSelect(select) {
      const slimSelect = new SlimSelect({ select });
      this.slimSelect = slimSelect;
      select.addEventListener("change", () => {
        this.slimSelect.render();
        this.element.dispatchEvent(new Event("input"));
      });
      this.element.addEventListener("input", (e) => {
        if (e.target !== this.element) {
          // avoid update search result when search in slimSelect suggestions
          e.stopPropagation();
        }
      });
      // hide the new element adding form
      this.element.querySelector("solid-form").classList.add("hidden");
      // triggered when a newly added element is saved
      this.element
        .querySelector("solid-form")
        .addEventListener("save", (saved) => {
          // append this element and select it
          this.slimSelect.setData([
            ...this.slimSelect.data.data,
            {
              text: saved.detail.resource.name,
              value: `{"@id": "${saved.detail.id}"}`,
            },
          ]);
          this.slimSelect.set(`{"@id": "${saved.detail.id}"}`);
        });

      // when data changes, re-build slimSelect
      if (this.mutationObserver) this.mutationObserver.disconnect();
      this.mutationObserver = new MutationObserver(() => {
        this.slimSelect.destroy();
        this.slimSelect = new SlimSelect({
          select,
          searchPlaceholder: this.element.attributes.placeholder.value,
          searchText: this.element.attributes["addable-no-result"].value,
          searchFilter: (option, filterValue) =>
            core.Helpers.fuzzyCompare(option.text, filterValue),
          addable: async (value) => {
            // when the add button is clicked, submit the form to create it
            this.element.querySelector('input[type="text"]').value = value;
            this.element.querySelector('input[type="submit"]').click();
          },
        });
      }).observe(select, {
        childList: true,
        characterData: true,
        subtree: true,
      });
    },
  };

  /**
   * Create and register a widget based on its tagName
   * @param tagName - string
   */
  const newWidgetFactory = (tagName) => {
    let widgetMixins = {
      templateMixin: core.Templates.formTemplates.dropdown,
      mixins: [
        core.TemplateAdditionMixins.templateAdditionDirectory.label,
        core.TemplateAdditionMixins.templateAdditionDirectory.addable,
        ...core.Templates.formTemplates.dropdown.dependencies,
        EventAddableMixin,
      ],
    };

    const newWidget = {
      // compose widget
      name: tagName,
      use: [
        ...widgetMixins.mixins,
        core.BaseWidgetMixin, // at the end so created() is called first
      ],
      get template() {
        return widgetMixins.templateMixin.template;
      },
    };

    core.Sib.register(newWidget); // and register component
  };

  newWidgetFactory("event-solid-form-dropdown-label-addable");
});
